pub enum ExtDataType {
Str,
Num,
Bool,
Timestamp,
Date,
Geom,
}Expand description
Externally visible data type variants for arguments and result types used and referenced by user-defined and registered functions invoked in filter expressions.
Variants
Str
A Unicode UTF-8 string.
Num
A numeric value including integers and floating points.
Bool
A boolean value.
Timestamp
An Instant with a granularity of a second or smaller. Timestamps are always in the time zone UTC (“Z”).
Date
An Instant with a granularity of a day. Dates are local without an associated time zone.
Geom
A spatial (geometry) value.