G

Enum G 

pub enum G {
    Null,
    Point(Point),
    Line(Line),
    Polygon(Polygon),
    Points(Points),
    Lines(Lines),
    Polygons(Polygons),
    Vec(Geometries),
    BBox(BBox),
}
Geometry type variants handled by this library.

Variants



Null

Undefined geometry.



Point(Point)

Point geometry.



Line(Line)

Line geometry.



Polygon(Polygon)

Polygon geometry.



Points(Points)

Point collection.



Lines(Lines)

Line collection.



Polygons(Polygons)

Polygon collection.



Vec(Geometries)

Mixed collection excluding BBOX.



BBox(BBox)

Bounding box geometry.

Implementations

Source

impl G

pub fn as_point(&self) -> Option<&Point>

Return this if it was indeed a Point, None otherwise.

pub fn as_line(&self) -> Option<&Line>

Return this if it was indeed a Line, None otherwise.

pub fn as_polygon(&self) -> Option<&Polygon>

Return this if it was indeed a Polygon, None otherwise.

pub fn as_points(&self) -> Option<&Points>

Return this if it was indeed a Point collection, None otherwise.

pub fn as_lines(&self) -> Option<&Lines>

Return this if it was indeed a Line collection, None otherwise.

pub fn as_polygons(&self) -> Option<&Polygons>

Return this if it was indeed a Polygon collection, None otherwise.