pub struct CRS { /* private fields */ }Expand description
Representation of a Coordinate Reference System
Implementations
Sourceimpl CRS
impl CRS
Sourcepub fn new(code: &str) -> Result<Self, MyError>
pub fn new(code: &str) -> Result<Self, MyError>
Try constructing a new instance and ensure that CRS has a non-trivial extent of validity which will be later used to validate geometry coordinates.
Sourcepub fn from_epsg(code: NonZero<usize>) -> Result<Self, MyError>
pub fn from_epsg(code: NonZero<usize>) -> Result<Self, MyError>
Construct a new instance from the given code assuming EPSG Authority if it’s a valid one (known by Proj).
Sourcepub fn check_point(&self, coord: &[f64]) -> Result<(), MyError>
pub fn check_point(&self, coord: &[f64]) -> Result<(), MyError>
Check if the given point coordinates are w/in the area-of-validity of this.