pub struct GPkgDataSource { /* private fields */ }Expand description
GeoPackage DataSource binding a .gpkg database file + a layer name that
maps rows to Features and Resources.
Implementations
Sourceimpl GPkgDataSource
impl GPkgDataSource
Sourcepub fn vtable(&self) -> String
pub fn vtable(&self) -> String
Return name of the virtual table created for querying this GeoPackage table.
This name is manufactured by pre-pending “vgpkg_” to the layer name
in a similar way to how spatialite handles GeoPackage files.
Sourcepub fn to_sql(&self, exp: &Expression) -> Result<String, MyError>
pub fn to_sql(&self, exp: &Expression) -> Result<String, MyError>
Transform given Expression to an SQL WHERE clause that can be used for selecting a subset of this data source items.