pub struct QString { /* private fields */ }Expand description
String based type used by Queryables to represent a plain string, and
a set of flags to indicate how to use it in case and/or accent insensitive
contexts.
Implementations