DataSource

Trait DataSource 

pub trait DataSource {
    // Required method
    fn srid(&self) -> Option<u32>;
}
Trait for a type that can act as a data source provider of Features and Resources, including a Geometry attribute, in the context of processing CQL2 filter expressions.

A CSV file, and a GeoPackage autonomous database file are examples of this.

Required Methods

fn srid(&self) -> Option<u32>

Return SRID of this if it was known at construction; None otherwise.

Implementors