IterableDS

Trait IterableDS 

pub trait IterableDS {
    type Item: TryInto<Resource, Error = Self::Err>;
    type Err;

    // Required method
    fn iter(
        &self,
    ) -> Result<impl Iterator<Item = Result<Self::Item, Self::Err>>, Self::Err>;
}
Capability of a DataSource to provide an iterator over a collection of Features or Resources.

Required Associated Types

type Item: TryInto<Resource, Error = Self::Err>

Type representing a Feature which must be convertible to a Resource.

type Err

Error raised by this.

Required Methods

fn iter( &self, ) -> Result<impl Iterator<Item = Result<Self::Item, Self::Err>>, Self::Err>

Return an iterator over this data source Features.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.

Implementors