pub enum Outcome {
T,
F,
N,
}Expand description
Possible outcome values when evaluating an Expression against an individual Resource from a collection.
From OGC CQL2:
Each resource instance in the source collection is evaluated against a filtering expression. The net effect of evaluating a filter Expression is a subset of resources that satisfy the predicate(s) in the Expression.
Logically connected predicates are evaluated according to the following
truth table, where T is TRUE, F is FALSE and N is NULL.
+-----+-----+---------+---------+
| P1 | P2 | P1 & P2 | P1 | P2 |
+-----+-----+---------+---------+
| T | T | T | T |
| T | F | F | T |
| F | T | F | T |
| F | F | F | F |
| T | N | N | T |
| F | N | F | N |
| N | T | N | T |
| N | F | F | N |
| N | N | N | N |
+-----+-----+---------+---------+Variants
T
The input satisfies the Expression and should be marked as being in the result set.
F
The input does not satisfy the filter Expression and should not be included the result set.
N
Likewise.
Implementations