pub enum Q {
Null,
Bool(bool),
Num(f64),
Str(QString),
Geom(Geometry),
Instant(Bound),
Interval(Bound, Bound),
List(Vec<Q>),
}Expand description
A Resource queryable property possible concrete value variants.
Variants
Null
Unknown or undefined w/in the current context.
Bool(bool)
A known boolean value.
Num(f64)
A known numeric literal.
Str(QString)
Either a known UTF8 character string literal, or one that when used in comparisons, should be used ignoring its case and/or accent(s).
Geom(Geometry)
A known geometry (spatial) instance.
Instant(Bound)
Either a known temporal instant or an unbounded value.
Interval(Bound, Bound)
A temporal interval.
List(Vec<Q>)
A list of other Queryables.
Implementations
Sourceimpl Q
impl Q
Sourcepub fn new_plain_str(value: &str) -> Self
pub fn new_plain_str(value: &str) -> Self
Create a new instance as a plain literal string from given argument after trimming it.
Sourcepub fn try_from_timestamp_str(value: &str) -> Result<Self, MyError>
pub fn try_from_timestamp_str(value: &str) -> Result<Self, MyError>
Try creating a new temporal timestamp variant instance from a string of the form fullDate followed by “T”, followed by utcTime.
Sourcepub fn try_from_timestamp_ns(value: i128) -> Result<Self, MyError>
pub fn try_from_timestamp_ns(value: i128) -> Result<Self, MyError>
Try creating a new temporal timestamp variant instance from a number of nanoseconds since the Unix epoch.
Sourcepub fn try_from_date_str(value: &str) -> Result<Self, MyError>
pub fn try_from_date_str(value: &str) -> Result<Self, MyError>
Try creating a new temporal date variant instance from a fullDate string.
IMPORTANT - CQL2 specs state that dates are to be considered as local wrt. time zones. This implementation however always assigns a UTC time zone.
Sourcepub fn try_from_date_ns(value: i128) -> Result<Self, MyError>
pub fn try_from_date_ns(value: i128) -> Result<Self, MyError>
Try creating a new temporal date variant instance from a number of nanoseconds since the Unix epoch.
Sourcepub fn try_from_wkt(value: &str) -> Result<Self, MyError>
pub fn try_from_wkt(value: &str) -> Result<Self, MyError>
Try creating a new instance from a Well Known Text encoded geometry.