pub trait DataSource { }Expand description
Marker trait for a type that can act as a data source provider of features and resources in the context of processing CQL2 filter expressions.
A CSV file, and a GeoPackage autonomous database file are examples of this.